/**
* GSVideo library.
* Processing library based on the gstremer-java bindings by Wayne Meissner:
* http://code.google.com/p/gstreamer-java/
*
* GSMovieMaker class for creating video files from screen pixels. Doesn't work.
* v0.3.11
* Changes:
* None
* Author: Andres Colubri
*/

/*
  Copyright (c) 2008 Andres Colubri

  This source is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This code is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  A copy of the GNU General Public License is available on the World
  Wide Web at <http://www.gnu.org/copyleft/gpl.html>. You can also
  obtain it by writing to the Free Software Foundation,
  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

package codeanticode.gsvideo;
import processing.core.*;

import java.io.*;
import java.nio.*;
import java.lang.reflect.*;
import java.util.concurrent.TimeUnit;
import java.util.Arrays;

import java.util.concurrent.SynchronousQueue;

import org.gstreamer.*;
import org.gstreamer.elements.*;
import org.gstreamer.Buffer;

/**
 * GSMovieMaker class.
 */
public class GSMovieMaker 
{
    int movieWidth;
    int movieHeight;
  
    ByteBuffer tmpBuffer;
    byte[] moviePixels;
  
    Pipeline gpipe;
    GSVideo gsvideo;
  
    private PApplet parent;
    int width, height;
  
    public static final int THEORA = 0;  
    public static final int XVID = 1;
/*
  public static final int WORST = 0;
  public static final int LOW = 1;
  public static final int MEDIUM = 2;
  public static final int HIGH = 3;
  public static final int BEST = 4;
  public static final int LOSSLESS = 5;
*/  
  
    int frameRate;
    int waitTime;
    SynchronousQueue<ByteBuffer> synchQueue;

    public GSMovieMaker(PApplet parent, int requestWidth, int requestHeight, String filename) 
    {
        this(parent, requestWidth, requestHeight, filename, THEORA, 30);
    }
  
    public GSMovieMaker(PApplet parent, int requestWidth, int requestHeight, String filename, int codecType, int ifps) 
    {
        this.parent = parent;
        //parent.super.init(parent.width, parent.height, PConstants.RGB);
	   
        gsvideo = new GSVideo();	   
        gsvideo.init();	   
	  
        // register methods
        parent.registerDispose(this);
	
        frameRate = ifps;
        Double t = 1000.0 / ifps;
        waitTime = t.intValue();
	
        width = parent.width;
        height = parent.height;
	
        synchQueue = new SynchronousQueue<ByteBuffer>();

        tmpBuffer = ByteBuffer.allocate(width * height * 4);
        movieWidth = requestWidth;
        movieHeight = requestHeight;
  
        gpipe = new Pipeline("GSMovieMaker");	  
	
        final GSBufferSrc buffersrc = new GSBufferSrc(parent, synchQueue, waitTime, movieWidth, movieHeight, "pixelbuffer source");
	
        final Element srcfilter = ElementFactory.make("capsfilter", "srcfilter");
        Caps fltcaps = new Caps("video/x-raw-rgb, width=" + movieWidth + ", height=" + movieHeight + ", bpp=32, depth=24, framerate=" + frameRate + "/1");
        srcfilter.setCaps(fltcaps);
	
        final Element videorate = ElementFactory.make("videorate", "videorate");
        final Element ratefilter = ElementFactory.make("capsfilter", "RateFilter");
        ratefilter.setCaps(Caps.fromString("video/x-raw-rgb, framerate=" + frameRate + "/1"));
	
        final Element conv = ElementFactory.make("ffmpegcolorspace", "ColorConverter");
	
        final Element videoenc;
        final Element muxer;
        if (codecType == THEORA) 
        {
            videoenc = ElementFactory.make("theoraenc",  "theoraenc");
            muxer = ElementFactory.make("oggmux", "oggmux");
        }
        else 
        {
            videoenc = ElementFactory.make("xvidenc",  "xvidenc");
            muxer = ElementFactory.make("avimux", "avimux");	
        }
	
        Element videosink = ElementFactory.make("filesink", "sink");
        videosink.set("location", parent.savePath(filename));
	
        gpipe.addMany(buffersrc, srcfilter, videorate, ratefilter, conv, videoenc, muxer, videosink);
        Element.linkMany(buffersrc, srcfilter, videorate, ratefilter, conv, videoenc, muxer, videosink);

        gpipe.setState(State.PLAYING);
    }
  
    public void addFrame() 
    {
        parent.loadPixels();
	
        // Converting int[] pixels array into byte array.
        tmpBuffer.rewind();
        tmpBuffer.asIntBuffer().put(parent.pixels);	

        try
        {	
            parent.print("Offering buffer...");
            synchQueue.offer(tmpBuffer, waitTime, TimeUnit.MILLISECONDS);
            parent.println("Sent buffer.");
        }
        catch (InterruptedException ie)
        {
            ie.printStackTrace();
        }
    }

    public void restart() 
    { 
        gpipe.setState(State.PLAYING);
    }
  
    public void finish() 
    {
        gpipe.setState(State.PAUSED);
    }
   
    public void dispose() 
    {
    } 
  
    static long NANOS_PER_SECOND = 1000000000;
}
